@@ -2,6 +2,12 @@ from .excel_module import get_df,get_row,get_cell_value,gpd,eatAll
 from .file_utils import safe_excel_save
 from shapely.geometry import Polygon,MultiPolygon
 import os
+class SingletonMeta(type):
+    _instances = {}
+    def __call__(cls, *args, **kwargs):
+        if cls not in cls._instances:
+            cls._instances[cls] = super(SingletonMeta, cls).__call__(*args, **kwargs)
+        return cls._instances[cls]
 def get_project_name(filePath):
     directory = os.path.dirname(filePath)
     baseName = os.path.basename(filePath)
@@ -16,28 +22,25 @@ def update_file(df,compFilePath,suffix):
     filePath = create_file_name(compFilePath,suffix)
     safe_excel_save(df,filePath)
     return filePath
-class landManager:
-    _instance = None
+class landManager(metaclass=SingletonMeta):
     def __init__(self,directory_js=None,default_polygon=None,column=None,value=None,epsg=4326,index=1):
-        directory_js = directory_js or {}
-        if landManager._instance is not None:
-            raise Exception("This class is a singleton!")
-        else:
-            landManager._instance = self
-        self.file_manager={}
-        self.closest_points={}
-        self.default_polygon= default_polygon
-        for directory_name,directory_path in directory_js.items():
-            self.file_manager[directory_name] = {} 
-            self.ext_js = {"shp":'.shp', "prj":'.prj', "cpg":'.cpg', "dbf":'.dbf', "shx":'.shx','geojson':'.geojson'}
-            for file_item in os.listdir(directory_path):
-                item_path = os.path.join(directory_path,file_item)
-                self.file_manager[directory_name][os.path.splitext(file_item)[-1][1:]] = item_path
-            if self.default_polygon == None:
-                gdf=self.get_contents(directory_name,file_type='shp')
-                if isinstance(gdf,gpd.GeoDataFrame):
-                    self.default_polygon = self.get_polygon(gdf,column=column,value=value,index=index)
-            self.check_geo_json(directory_name,epsg=epsg)
+        if not hasattr(self, 'initialized'):
+            self.initialized = True
+            directory_js = directory_js or {}
+            self.file_manager={}
+            self.closest_points={}
+            self.default_polygon= default_polygon
+            for directory_name,directory_path in directory_js.items():
+                self.file_manager[directory_name] = {} 
+                self.ext_js = {"shp":'.shp', "prj":'.prj', "cpg":'.cpg', "dbf":'.dbf', "shx":'.shx','geojson':'.geojson'}
+                for file_item in os.listdir(directory_path):
+                    item_path = os.path.join(directory_path,file_item)
+                    self.file_manager[directory_name][os.path.splitext(file_item)[-1][1:]] = item_path
+                if self.default_polygon == None:
+                    gdf=self.get_contents(directory_name,file_type='shp')
+                    if isinstance(gdf,gpd.GeoDataFrame):
+                        self.default_polygon = self.get_polygon(gdf,column=column,value=value,index=index)
+                self.check_geo_json(directory_name,epsg=epsg)
     @staticmethod
     def get_instance(directory_js=None):
         if landManager._instance is None:
@@ -66,7 +69,8 @@ class landManager:
             filePath = update_file(df,file_path,suffix)
             if filePath:
                 self.file_manager[dir_name][suffix] = filePath
-        return filePath
+            return filePath
+        return file_path
     def get_file_path(self,dir_name,file_type):
         if dir_name == None:
             dir_name = list(self.file_manager.keys())
@@ -78,7 +82,7 @@ class landManager:
         if file_path and os.path.isfile(file_path):
             return file_path
     def get_contents(self,dir_name,file_type,update=None):
-        if not isinstance(update,bool):
+        if update is not None and not isinstance(update,bool):
             file_path = self.update_file(update,dir_name,file_type)
         if os.path.isfile(file_type):
             file_path = file_type
