@@ -1,16 +1,43 @@
 from odf.table import Table, TableRow, TableCell
 from odf.opendocument import load
-from .excel_module import *
+import pandas as pd
+import geopandas as gpd
+from .proj_tools import *
+from abstract_utilities import *
 from odf import text, teletype
 from datetime import datetime
-import tempfile,shutil,os,ezodf,inspect,logging
+from openpyxl import load_workbook, utils
+from difflib import get_close_matches
+from werkzeug.utils import secure_filename
+from werkzeug.datastructures import FileStorage
+import tempfile,shutil,os,ezodf,inspect,logging,logging
+from .general_functions import split_and_clean_lines,make_type,convert_column,return_float_or_int,safe_get,get_number
 logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')
 def source_ext(typ=None):
     source_js = {'.parquet':'pyarrow','.txt':'python','.xlsx':'openpyxl','.xls':'openpyxl','.xlsb':'pyxlsb','.ods':'odf','.geojson':'GeoJSON'}
     if typ:
         source_js = source_js.get(typ)
     return source_js
-
+def is_file(file_path):
+    if file_path and isinstance(file_path, str) and os.path.isfile(file_path):
+        return os.path.splitext(file_path)[-1].lower()
+def isDataFrame(obj):
+    return isinstance(obj, pd.DataFrame)
+def create_dataframe(new_data=None,columns=None):
+    if isDataFrame(new_data):
+        return new_data
+    new_data = new_data or {}
+    if isinstance(new_data,dict):
+        new_data=[new_data]
+        if columns == None:
+            columns=[]
+            for datas in new_data:
+                if isinstance(datas,dict):
+                    columns=list(set(columns+list(datas.keys())))
+        if columns ==False:
+            columns=None
+    if isinstance(new_data,list):
+        return pd.DataFrame(new_data,columns=columns)
 def read_ods_file(file_path):
     doc = ezodf.opendoc(file_path)
     sheets = {}
@@ -35,6 +62,100 @@ def read_ods_file(file_path):
 def read_ods(file_path,xlsx_path=None):
     ods_to_xlsx(file_path,xlsx_path)
     return pd.read_excel(xlsx_path)
+def filter_df(df, nrows=None, condition=None, indices=None):
+    """
+    Apply filtering to a DataFrame based on specified criteria.
+
+    Parameters:
+    - df (DataFrame): The DataFrame to filter.
+    - nrows (int, optional): Number of rows to return from the start.
+    - condition (pd.Series, optional): Boolean series for row filtering.
+    - indices (list of int, optional): Row indices to select.
+
+    Returns:
+    - DataFrame: Filtered DataFrame.
+    """
+    if nrows is not None:
+        df = df.head(nrows)
+    if condition is not None:
+        df = df[condition]
+    if indices is not None:
+        df = df.iloc[indices]
+    return df
+def read_shape_file(source):
+    file_ext = is_file(source)
+    if file_ext:
+        if file_ext in ['.shp', '.cpg', '.dbf', '.shx','.geojson']:
+            df = gpd.read_file(source,driver=source_ext(file_ext))
+            return df
+        elif file_ext in ['.prj']:
+            df = get_wkt_string(source)
+            return df
+    return False
+def get_df(source=None, nrows=None, skiprows=None, condition=None, indices=None):
+    """
+    Load a DataFrame from various sources with optional filtering.
+
+    Parameters:
+    - source (str, pd.DataFrame, gpd.GeoDataFrame, dict, list, FileStorage): Source of the data.
+    - nrows (int, optional): Number of rows to load.
+    - header (int, list of int, 'infer', optional): Row(s) to use as the header.
+    - skiprows (list-like, int, optional): Rows to skip at the beginning.
+    - condition (pd.Series, optional): Condition for filtering rows.
+    - indices (list of int, optional): Indices of rows to select.
+
+    Returns:
+    - pd.DataFrame or gpd.GeoDataFrame: Loaded and optionally filtered data.
+    """
+    if isinstance(source, (pd.DataFrame, gpd.GeoDataFrame)):
+        return filter_df(source, nrows=nrows, condition=condition, indices=indices)
+
+    if source is None:
+        logging.error("No source provided for loading data.")
+        return create_dataframe()
+    file_ext = is_file(source)
+    if file_ext:
+        
+       
+            logging.info(f"Loading data from file with extension: {file_ext}")
+            if file_ext in ['.csv', '.tsv', '.txt']:
+                sep = {'csv': ',', 'tsv': '\t'}.get(file_ext.strip('.'), None)
+                df = pd.read_csv(source,  skiprows=skiprows, sep=sep, nrows=nrows)
+            elif file_ext in ['.ods', '.xlsx', '.xls', '.xlsb']:
+                engine = {'ods': 'odf', 'xlsx': 'openpyxl', 'xls': 'xlrd', 'xlsb': 'pyxlsb'}.get(file_ext.strip('.'))
+                df = pd.read_excel(source,  skiprows=skiprows, engine=engine, nrows=nrows)
+            elif file_ext == '.json':
+                df = pd.read_json(source, nrows=nrows)
+            elif file_ext == '.parquet':
+                df = pd.read_parquet(source, nrows=nrows)
+            elif file_ext in ['.shp', '.cpg', '.dbf', '.shx','.geojson','.prj']:
+                df = read_shape_file(source)
+            else:
+                try:
+                    df = read_from_file(source)
+                except:                
+                    raise logging.info(f"Unsupported file extension: {file_ext}")
+            if not isinstance(df, (dict, list,FileStorage)):
+                return filter_df(df, nrows=nrows, condition=condition, indices=indices)
+            source = df
+
+
+    if isinstance(source, FileStorage):
+        try:
+            logging.info(f"Reading from FileStorage: {secure_filename(source.filename)}")
+            df = pd.read_excel(source.stream, nrows=nrows)
+            return filter_df(df, nrows=nrows, condition=condition, indices=indices)
+        except Exception as e:
+            logging.error(f"Failed to read from FileStorage: {e}")
+            return None
+
+    if isinstance(source, (dict, list)):
+        logging.info("Creating DataFrame from in-memory data structure.")
+        df = pd.DataFrame(source)
+        return filter_df(df, nrows=nrows, condition=condition, indices=indices)
+
+    logging.error("Invalid data source type provided.")
+    return None
 def save_df(df,file_path,index=None,suffix=None,engine=None):
     df = get_df(df)
     suffix = suffix or os.path.splitext(file_path)[-1] or '.xlsx'
@@ -47,6 +168,8 @@ def save_df(df,file_path,index=None,suffix=None,engine=None):
             df.to_csv(file_path,  engine=engine or source_ext(suffix))
         elif suffix in [".shp", ".cpg", ".dbf", ".shx",'.geojson']:
             df.to_file(file_path,driver=source_ext(suffix))
+        elif suffix in ['.prj']:
+            save_valid_crs(df, file_path)
         elif suffix == '.parquet':
             df.to_parquet(file_path, engine=engine or source_ext(suffix))
         else:
@@ -63,12 +186,15 @@ def save_df(df,file_path,index=None,suffix=None,engine=None):
     except Exception as e:
         logging.info(f"Failed to read file: {e}")
         return False
-    
+    return True
     
 def safe_excel_save(df,original_file_path,index=False,suffix=None,engine=None):
+    suffix = suffix or os.path.splitext(original_file_path)[-1] or '.xlsx'
+    result = None
     with tempfile.NamedTemporaryFile(delete=False, suffix=suffix) as tmp:
         temp_file_name = tmp.name
-        result = save_df(df,original_file_path,index=index,suffix=suffix,engine=engine)  # Save your DataFrame to the temp file
+        result = save_df(df,temp_file_name,index=index,suffix=suffix,engine=engine)  # Save your DataFrame to the temp file
+        logging.info(f"{temp_file_name} {os.path.isfile(temp_file_name)},{os.path.getsize(temp_file_name)}")
     if result and os.path.getsize(temp_file_name) > 0:
         shutil.move(temp_file_name, original_file_path)
     else:
