@@ -2,7 +2,7 @@ from abstract_security import *
 import os, time
 from datetime import datetime, timedelta
 from itertools import permutations
-
+from abstract_utilities import eatAll
 def capitalize(string):
    string = str(string)
    return  string[0].upper()+string[1:]
@@ -124,6 +124,13 @@ def all_combinations_of_strings(strings):
     all_perms = permutations(strings)
     combinations = [''.join(perm) for perm in all_perms]
     return combinations
+def clean_list(list_obj):
+    while '' in list_obj:
+        list_obj.remove('')
+    return list_obj
+def split_and_clean_lines(query):
+    query = query.replace('\n',',')
+    return list(set([eatAll(quer,[',',' ','\t','\n']) for quer in clean_list(query.split(','))]))
 
 def all_date_formats():
     date_formats = []
